#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# 模块重写脚本

import os
from ruamel import yaml
from tdf_tools.tools.dependencies_analysis import DependencyAnalysis
from tdf_tools.tools.print import Print
from tdf_tools.tools.module.module_tools import ModuleTools
from tdf_tools.tools.shell_dir import ShellDir


class ModuleDependenciesRewriteUtil:
    def __init__(self):
        Print.debug("开始进行依赖重写")
        self.moduleJsonData = ModuleTools.getModuleJsonData()
        self.moduleNameList = ModuleTools.getModuleNameList()
        self.initJsonData = ModuleTools.getInitJsonData()

    # 分析lock文件，获取所有的packages
    def _analysisLock(self):
        os.chdir(self.__moduleGenPath)
        # 分析前先执行pub upgrade
        os.popen("flutter pub upgrade").read()

        # 读取lock内容
        with open("pubspec.lock", encoding="utf-8") as f:
            doc = yaml.round_trip_load(f)
            if isinstance(doc, dict) and doc.__contains__("packages"):
                f.close()
                return doc["packages"]

    # 是否是壳模块
    def _isShellModule(self):
        return self.moduleJsonData[self.__moduleName]["type"] == "app"

    # 确认哪些依赖需要重写
    def _confirmRewriteDependencies(self, isShell):
        if isShell:  # 壳模块重写所有配置的依赖
            for item in self.moduleNameList:
                if item != self.__moduleName:  # 壳自己不加入重写列表
                    self.__needRewriteDependencies.append(item)
        else:  # 如果不是壳模块，则进行lock文件内的package列表和开发模块匹配，匹配上则添加到override列表
            for package in self.__moduleDependenciesMap:
                for module in self.moduleNameList:
                    if package == module:
                        self.__needRewriteDependencies.append(module)

        Print.stage("{0}中以下依赖将被override".format(self.__moduleName))
        Print.debug(self.__needRewriteDependencies)

    def _addOverrideDependencies(self):
        mDict = dict()
        for item in self.__needRewriteDependencies:
            mDict[item] = {"path": "../{0}/".format(item)}

        return mDict

    # 添加dependency_overrides

    def _rewriteDependencies(self, isShell):
        os.chdir(self.__moduleGenPath)
        with open("pubspec.yaml", encoding="utf-8") as f:
            doc = yaml.round_trip_load(f)
            if isinstance(doc, dict):
                self._confirmRewriteDependencies(isShell)
                if (
                    doc.__contains__("dependency_overrides")
                    and doc["dependency_overrides"] is not None
                ):
                    doc["dependency_overrides"] = None

                # 重写依赖
                overrideDict = dict()
                for item in self.__needRewriteDependencies:
                    if isShell:
                        overrideDict[item] = {
                            "path": "../.tdf_flutter/{0}/".format(item)
                        }
                    else:
                        overrideDict[item] = {"path": "../{0}/".format(item)}
                if len(self.__needRewriteDependencies) > 0:
                    doc["dependency_overrides"] = overrideDict

                with open("pubspec.yaml", "w+", encoding="utf-8") as reW:
                    yaml.round_trip_dump(
                        doc,
                        reW,
                        default_flow_style=False,
                        encoding="utf-8",
                        allow_unicode=True,
                    )
                    reW.close()
                    # 依赖重写完，执行pub upgrade更新lock文件
                    os.popen("flutter pub upgrade").read()
                    Print.debug("lock文件已更新")
            f.close()

    # 添加dependency_overrides，如果已存在模块的override，则不修改

    def _rewriteDependenciesIfUpdate(self, isShell):
        os.chdir(self.__moduleGenPath)
        with open("pubspec.yaml", encoding="utf-8") as f:
            doc = yaml.round_trip_load(f)
            if isinstance(doc, dict):
                self._confirmRewriteDependencies(isShell)

                global existUpdate
                existUpdate = False

                # 删除不存在于重写依赖列表中的key
                if (
                    doc.__contains__("dependency_overrides")
                    and doc["dependency_overrides"] is not None
                    and isinstance(doc["dependency_overrides"], dict)
                ):
                    keyList = list(doc["dependency_overrides"].keys())
                    for item in keyList:
                        if item not in self.__needRewriteDependencies:
                            Print.debug("${0}依赖被删除".format(item))
                            existUpdate = True
                            del doc["dependency_overrides"][item]

                if doc.__contains__("dependency_overrides"):
                    overrideDict = doc["dependency_overrides"]
                    if overrideDict is None:
                        overrideDict = dict()
                else:
                    overrideDict = dict()
                for item in self.__needRewriteDependencies:
                    if (
                        doc.__contains__("dependency_overrides")
                        and doc["dependency_overrides"] is not None
                        and isinstance(doc["dependency_overrides"], dict)
                        and doc["dependency_overrides"].get(item, -1) != -1
                    ):
                        Print.debug("${0}依赖已存在（不予修改）".format(item))
                    else:
                        Print.debug("${0}依赖不存在（添加）".format(item))
                        existUpdate = True
                        if isShell:
                            overrideDict[item] = {
                                "path": "../.tdf_flutter/{0}/".format(item)
                            }
                        else:
                            overrideDict[item] = {"path": "../{0}/".format(item)}
                doc["dependency_overrides"] = overrideDict
                if existUpdate:
                    with open("pubspec.yaml", "w+", encoding="utf-8") as reW:
                        yaml.round_trip_dump(
                            doc,
                            reW,
                            default_flow_style=False,
                            encoding="utf-8",
                            allow_unicode=True,
                        )
                        reW.close()
                        # 依赖重写完，执行pub upgrade更新lock文件
                        os.popen("flutter pub upgrade").read()
                        Print.debug("lock文件已更新")
                else:
                    Print.debug("yaml无更新")

            f.close()

    # 重写依赖 本地依赖

    def rewrite(self, reWriteOnlyChange=False):
        for module in self.moduleNameList:
            Print.title("模块{0}依赖分析..".format(module))
            ShellDir.goInTdfFlutterDir()
            os.chdir(module)
            self.__moduleGenPath = os.getcwd()
            # 这一步会生成lock文件
            self.__moduleDependenciesMap = self._analysisLock()

        Print.title("壳依赖分析..")
        ShellDir.goInShellDir()
        self.__moduleGenPath = os.getcwd()
        self.__moduleDependenciesMap = self._analysisLock()

        # 分析依赖树，从下至上之行upgrade
        dependencyAnalysis = DependencyAnalysis()
        dependencyList = dependencyAnalysis.getDependencyOrder()

        moduleLength = len(dependencyList)
        for module in dependencyList[: moduleLength - 1]:
            self.__moduleName = module
            ShellDir.goInTdfFlutterDir()
            os.chdir(module)
            self.__moduleGenPath = os.getcwd()
            self.__moduleDependenciesMap = self._analysisLock()
            self.__needRewriteDependencies = []
            if reWriteOnlyChange:
                self._rewriteDependenciesIfUpdate(isShell=False)
            else:
                self._rewriteDependencies(isShell=False)
        ShellDir.goInShellDir()
        self.__moduleName = self.initJsonData["shellName"]
        self.__moduleGenPath = os.getcwd()
        self.__moduleDependenciesMap = self._analysisLock()
        self.__needRewriteDependencies = []
        if reWriteOnlyChange:
            self._rewriteDependenciesIfUpdate(isShell=True)
        else:
            self._rewriteDependencies(isShell=True)



from enum import Enum
import os
import platform


class Platform(Enum):
    MACOS = 1
    WINDOWS = 2
    LINUX = 3


class PlatformTools:
    def pwd_cmd() -> str:
        if PlatformTools.is_windows():
            return "cd"
        else:
            return "pwd"

    def is_windows() -> bool:
        return PlatformTools.platform() == Platform.WINDOWS

    def platform():
        if platform.system().lower() == "windows":
            return Platform.WINDOWS
        elif platform.system().lower() == "macos":
            return Platform.MACOS
        elif platform.system().lower() == "linux":
            return Platform.LINUX
        else:
            return Platform.MACOS

    # 获取当前目录
    def curPath():
        cmd = "pwd"
        mPlatform = PlatformTools.platform()
        if mPlatform == Platform.WINDOWS:
            cmd = "cd"

        return os.popen(cmd).read().split("\n")[0]



import os
import shutil
from tdf_tools.tools.cli.module_deps_rewrite import ModuleDependenciesRewriteUtil
from tdf_tools.tools.config.config import CLIJsonConfig
from tdf_tools.tools.config.initial_json_config import InitialJsonConfig
from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.gitlab.gitlab_utils import GitlabUtils
from tdf_tools.tools.gitlab.python_gitlab_api import GitlabAPI
from tdf_tools.tools.print import Print
from tdf_tools.tools.shell_dir import ShellDir


class ProjectCLI(object):
    def initial(self):
        ShellDir.goInShellDir()
        if os.path.exists("tdf_cache"):
            Print.debug("tdf_cache中已存在项目配置文件。继续执行init命令会将tdf_cache和.tdf_flutter删除。")
            if input("你确定要继续执行init命令吗？(y/n)") == "y":
                if input("真的确定了吗？(y/n)") == "y":
                    if input("最后一遍，真的吗，不后悔？(y/n)") == "y":
                        if os.path.exists("tdf_cache"):
                            shutil.rmtree(r"tdf_cache")
                        ShellDir.goInShellDir()
                        os.chdir(os.path.abspath(r".."))
                        if os.path.exists(".tdf_flutter"):
                            shutil.rmtree(r".tdf_flutter")
                        Print.error("d")
                    else:
                        Print.error("exit")
                else:
                    Print.error("exit")
            else:
                Print.error("exit")
        ShellDir.goInShellDir()
        self.initInitialJson()
        self.cliDeps()

    def initInitialJson(self):
        moduleJson = CLIJsonConfig.getModuleConfig()
        ShellDir.goInShellDir()

        featureBranch = Cmd.run("git rev-parse --abbrev-ref HEAD").replace("\n", "")

        shellModule = ShellDir.getModuleNameFromYaml("pubspec.yaml")

        # 初始化开发的模块列表
        moduleList = []
        while True:
            moduleName = input("输入需要开发的模块名:(添加:+name，删除:-name，完成输入:!):")
            if moduleName != "":
                if moduleName == "!" or moduleName == "！":
                    break
                if moduleJson.get(moduleName[1:], -1) != -1:
                    if moduleName[0] == "+":
                        moduleList.append(moduleName[1:])
                        print("模块{0}已添加".format(moduleName[1:]))
                    elif moduleName[0] == "-":
                        moduleList.remove(moduleName[1:])
                        print("模块{0}已移除".format(moduleName[1:]))
                else:
                    if moduleName[0] == "+":
                        print("未从仓库中找到模块{0}，添加失败".format(moduleName[1:]))
                    elif moduleName[0] == "-":
                        print("已添加模块列表中未找到模块{0}".format(moduleName[1:]))
                moduleList = list(dict.fromkeys(moduleList))

                print("已选模块列表：{0}".format(moduleList))

        # 写入项目环境配置文件
        CLIJsonConfig.saveInConfig(
            featureBranch=featureBranch, shellModule=shellModule, moduleList=moduleList
        )

    # deps
    def cliDeps(self):
        config = InitialJsonConfig()
        featureBranch = config.featureBranch
        moduleNameList = config.moduleNameList
        moduleJsonData = CLIJsonConfig.getModuleConfig()

        # 数据校验
        self.validateConfig(moduleNameList, moduleJsonData)

        gitlabUtils = GitlabUtils()

        gitlabApi = GitlabAPI()

        for module in moduleNameList:
            Print.title(module)
            ShellDir.goInTdfFlutterDir()
            gitlabUtils.clone(module, moduleJsonData[module]["git"])
            os.chdir(module)
            gitlabApi.createBranch(moduleJsonData[module]["id"], featureBranch)
            gitlabUtils.pull()
            gitlabUtils.checkout(featureBranch)
        # 依赖 重写
        reWrite = ModuleDependenciesRewriteUtil()
        reWrite.rewrite()

    # 校验配置是否正确，所有需要开发的库是否存在于模块配置json文件中
    def validateConfig(self, moduleNameList, moduleJsonData):
        if isinstance(moduleJsonData, dict):
            for module in moduleNameList:
                if module not in moduleJsonData.keys():
                    Print.error(
                        "配置的开发模块{0}没有找到git仓库信息。请确保 1. 模块名配置正确； 2. 执行 tdf_tools module-update 更新git信息配置文件".format(
                            module
                        )
                    )



#!/usr/bin/env python
# encoding=utf-8

import json
import os
import subprocess
import copy
from ruamel import yaml
import requests
from tdf_tools.tools.dependencies_analysis import DependencyAnalysis
from tdf_tools.tools.config.initial_json_config import InitialJsonConfig
from tdf_tools.tools.shell_dir import ShellDir

PRIVATE_REPOSITORY_HOST = "http://10.1.85.141"

# 版本号自增 依赖修改

# 读取当前文件夹内的pubspec.yaml文件


def yamlLoad():
    # 读取pubsepc.yaml内容
    with open('pubspec.yaml', encoding='utf-8') as f:
        doc = yaml.round_trip_load(f)
        if (isinstance(doc, dict)):
            f.close()
            return doc

# 规范版本号


def strictVersion(version):
    if isinstance(version, str):
        tagList = version.split('.')
        if len(tagList) == 3 and int(tagList[1]) <= 9:
            if len(tagList[2]) == 2:
                return version
            elif len(tagList[2]) == 1:
                return version + '1'
            else:
                print('❌ tag号规范检测失败，请严格按照规范修改tag号,例：x.y.z, （0<=x, 0<=y<=9, 00<=z<=99）')
                exit(-1)
        else:
            print('❌ tag号规范检测失败，请严格按照规范修改tag号,例：x.y.z, （0<=x, 0<=y<=9, 00<=z<=99）')
            exit(-1)

# 获取已经发布过的远程仓库列表


def getRemoteRepositories():
    try:
        r = requests.get(
            PRIVATE_REPOSITORY_HOST + '/webapi/packages?size=1000')
        mDict = json.loads(r.text)
        mRepositoryMap = {}
        for item in mDict['data']['packages']:
            mRepositoryMap[item['name']] = item['latest']
        return mRepositoryMap
    except:
        print('❌ 读取私库内模块列表失败，请确保私库正常运行')
        exit(-1)

# 版本号自增, tagNumber = 1019


def addVersion(tagNumber, shouldAdd=True):
    try:
        if shouldAdd:
            tagNumber = tagNumber + 1
        upgradeTag = "{0}.{1}.{2}".format(int(
            tagNumber / 1000), int((tagNumber % 1000) / 100), "{:0>2d}".format(tagNumber % 100))
        print("对{0}进行自增,自增后为{1}".format(tagNumber - 1, upgradeTag))
        return upgradeTag
    except Exception as ex:
        print('❌ 版本号自增失败:{0}'.format(ex))
        exit(-1)

# 版本号比对


def getUpgradeVersion(currentVersion, remoteVersion):
    try:
        currentVersionList = currentVersion.split('.')
        currentVersionNumber = int(currentVersionList[0]) * 1000 + int(
            currentVersionList[1]) * 100 + int(currentVersionList[2])

        # 如果远端版本为空，则直接对当前版本进行自增操作即可
        if remoteVersion == None:
            return addVersion(currentVersionNumber)

        remoteVersionList = remoteVersion.split('.')
        remoteVersionNumber = int(remoteVersionList[0]) * 1000 + int(
            remoteVersionList[1]) * 100 + int(remoteVersionList[2])

        if currentVersionNumber > remoteVersionNumber:
            return addVersion(currentVersionNumber, shouldAdd=False)

        if currentVersionNumber > remoteVersionNumber:
            return addVersion(currentVersionNumber)
        else:
            return addVersion(remoteVersionNumber)
    except Exception as ex:
        print('❌ 版本号比对出现异常:{0}'.format(ex))
        exit(-1)


def tagInfoObj():

    analysis = DependencyAnalysis()
    treeJson = json.loads(analysis.generate())
    print(treeJson)

    remoteRepositoryMap = getRemoteRepositories()
    initJsonData = InitialJsonConfig()

    result = {}

    for item in initJsonData.moduleNameList:
        ShellDir.goInTdfFlutterDir()
        os.chdir(item)
        print("检测模块：{0}".format(item))

        yamlInfo = yamlLoad()
        if yamlInfo.__contains__('version') is not True:
            print("❌模块{0}yaml文件中没有检测到version版本号字段，请设置".format(item))
            exit(-1)

        current = strictVersion(yamlInfo['version'])

        remote = None
        if remoteRepositoryMap.__contains__(item):
            remote = remoteRepositoryMap[item]

        result[item] = {
            "remote": remote,
            "current": current,
            "upgrade": getUpgradeVersion(current, remote),
            'sort': 0
        }

    for item in result:
        for index in range(len(treeJson)):
            if treeJson[index].__contains__(item):
                result[item]['sort'] = index

    exit(json.dumps(result))


# 修改依赖
def changeVersionAndDependencies(jsonData):

    # 例："{\"tdf_presell\":{\"remote\":\"1.0.05\",\"current\":\"1.0.04\",\"upgrade\":\"1.0.06\",\"sort\":1},\"tdf_goods_videos_manager\":{\"remote\":\"1.0.06\",\"current\":\"1.0.05\",\"upgrade\":\"1.0.07\",\"sort\":1},\"tdf_widgets\":{\"remote\":\"1.0.12\",\"current\":\"1.0.12\",\"upgrade\":\"1.0.13\",\"sort\":0}}"
    # tempStr = "{\"tdf_presell\":{\"remote\":\"1.0.05\",\"current\":\"1.0.04\",\"upgrade\":\"1.0.06\",\"sort\":1},\"tdf_goods_videos_manager\":{\"remote\":\"1.0.06\",\"current\":\"1.0.05\",\"upgrade\":\"1.0.07\",\"sort\":1},\"tdf_widgets\":{\"remote\":\"1.0.12\",\"current\":\"1.0.12\",\"upgrade\":\"1.0.13\",\"sort\":0}}"
    # jsonData = json.loads(tempStr)

    initJsonData = InitialJsonConfig()
    for item in initJsonData.moduleNameList:
        ShellDir.goInTdfFlutterDir()
        os.chdir(item)
        print("\n修改模块版本号和依赖：{0}".format(item))

        yamlInfo = yamlLoad()
        # dependency_overrides依赖
        dependencyOverridesDict = dict
        # dependencies依赖
        dependencyDict = dict
        if yamlInfo.__contains__('dependency_overrides'):
            dependencyOverridesDict = copy.deepcopy(
                yamlInfo['dependency_overrides'])

        if yamlInfo.__contains__('dependencies'):
            dependencyDict = copy.deepcopy(
                yamlInfo['dependencies'])

        # dependencyOverridesDict不为空，才需要修改dependencyDict
        # 修改规则：1.dependency中没有，dependency_overrides中有，则添加为tdg依赖方式
        #         2.dependency中有，dependency_overrides中有，则覆盖为tag依赖方式
        if dependencyOverridesDict is not None and isinstance(dependencyOverridesDict, dict) and dependencyOverridesDict.__len__() > 0:
            for dependencyOverridesItem in dependencyOverridesDict:
                upwardReference = True
                if dependencyOverridesItem in dependencyDict:
                    if isinstance(dependencyDict[dependencyOverridesItem], dict) and 'version' in dependencyDict[dependencyOverridesItem]:
                        itemDict = dependencyDict[dependencyOverridesItem]
                        if isinstance(itemDict, dict) and 'version' in itemDict:
                            versionStr = itemDict["version"]
                            if isinstance(versionStr, str) and versionStr.__contains__('^'):
                                upwardReference = True

                print('--- 修改节点{0}的依赖 如下'.format(dependencyOverridesItem))

                dependencyDict[dependencyOverridesItem] = {
                    "hosted": {
                        "name": "{0}".format(dependencyOverridesItem),
                        "url": "http://10.1.85.141"
                    },
                    "version": "{0}{1}".format('^' if upwardReference else '', jsonData[item]['upgrade'])
                }
                # 输出修改后的节点信息
                print(json.dumps(dependencyDict[dependencyOverridesItem]))

            # 重写
            try:
                yamlInfo['version'] = jsonData[item]['upgrade']
                del yamlInfo['dependency_overrides']
                yamlInfo['dependencies'] = dependencyDict

                with open('pubspec.yaml', 'w+', encoding='utf-8') as reW:
                    yaml.round_trip_dump(
                        yamlInfo, reW, default_flow_style=False, encoding='utf-8', allow_unicode=True)
                    reW.close()
            except Exception as e:
                print('重写时出现异常，错误原因：{0}'.format(e))
                exit(-1)


def createAndPushTag(tagModuleList):
    initJsonData = InitialJsonConfig()

    for moduleName in tagModuleList:

        isExist = False
        for item in initJsonData.moduleNameList:
            if moduleName == item:
                isExist = True
                ShellDir.goInTdfFlutterDir()
                os.chdir(item)
                print("\n{0}模块开始执行tag操作...".format(item))

                # 在yaml中获version，作为tag
                yamlInfo = yamlLoad()
                tag = yamlInfo['version']
                print("获取tag号：{0}".format(tag))

                print('exec: git tag {0}'.format(tag))
                res = os.popen('git tag {0}'.format(tag)).read()
                print(res)

                print('exec: git push origin {0}'.format(tag))
                res = os.popen('git push origin {0}'.format(tag)).read()
                print(res)

        if isExist is False:
            print('模块{0}没有找到'.format())
            exit(-1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tdf_tools',
 'tdf_tools.pipelines',
 'tdf_tools.tools',
 'tdf_tools.tools.cli',
 'tdf_tools.tools.config',
 'tdf_tools.tools.gitlab',
 'tdf_tools.tools.module',
 'tdf_tools.tools.package',
 'tdf_tools.tools.translate',
 'tdf_tools.tools.vscode']

package_data = \
{'': ['*']}

install_requires = \
['environs>=9.5.0,<10.0.0',
 'fire>=0.4.0,<0.5.0',
 'googletrans==3.1.0a0',
 'python-gitlab==1.15.0',
 'requests==2.27.1',
 'ruamel.yaml>=0.17.21,<0.18.0']

entry_points = \
{'console_scripts': ['tdf_tools = tdf_tools.app:main']}

setup_kwargs = {
    'name': 'tdf-tools',
    'version': '2.0.29',
    'description': '二维火 flutter 脚手架工具',
    'long_description': '## 帮助文档\n\n```shell\nNAME\n    tdf_tools - 二维火 Flutter 脚手架工具，包含项目构建，依赖分析，git等功能。。\n\nSYNOPSIS\n    tdf_tools GROUP | COMMAND\n\nDESCRIPTION\n    二维火 Flutter 脚手架工具，包含项目构建，依赖分析，git等功能。。\n\nGROUPS\n    GROUP is one of the following:\n\n     module\n       模块相关工具： tdf_tools module -h 查看详情\n\n     package\n       封包工具相关：tdf_tools package -h 查看详情\n\n     translate\n       国际化相关：tdf_tools translate -h 查看详情\n\nCOMMANDS\n    COMMAND is one of the following:\n\n     git\n       tdf_tools git【git 命令】：批量操作 git 命令, 例如 tdf_tools git push\n\n     router\n       tdf_tools router：会以交互式进行路由操作，对指定的模块执行路由生成和路由注册逻辑\n\n     upgrade\n       tdf_tools upgrade：升级插件到最新版本\n        \n```\n\n\n\n## 插件安装方式\n\n安装python包\n\n```\npip3 install tdf-tools --user\n```\n\n安装并更新python包\n\n```\npip3 install --upgrade tdf-tools --user\n```\n\n安装测试环境python包\n\n```\npip3 install -i https://test.pypi.org/simple/ tdf-tools --user\n```\n\n安装并更新测试环境python包\n\n```\npip3 install --upgrade -i https://test.pypi.org/simple/ tdf-tools --user\n```\n\n\n\n## 工具使用流程说明\n\n### 1.准备工作\n\n- 确保python的bin插件目录已经被配置到环境变量中（这一步不满足的话，插件安装之后是无法识别到本插件命令的）\n\n- 在~目录下，创建.tdf_tools_config文件并配置相关必需属性如下\n\n```json\ngit_private_token=***\n```\n\ngit_private_token是gitlab的token\n\n获取途径：进入gitlab页面，点击右上角头像，选择Preferences，选择左侧列表中的AccessToken进行创建\n\n**上述步骤如果没有做，会在使用插件时，会有提示**\n\n### 2.初始化\n\n#### i.进入壳目录（确保执行命令在壳目录内）\n\n#### ii.执行tdf_tools module init\n\n- 判断当前目录是否存在tdf_cache，若不存在，则会自动创建tdf_cache\n- 自动读取当前壳模块名称，写入initial_config.json配置文件；\n- 读取当前壳分支，写入initial_config.json配置文件；\n- 交互式提示用户输入需要开发的模块名并写入initial_config.json配置文件的moduleNameList列表字段中\n- ！退出，即输入完成\n- 自动clone所有开发模块到  ```../.tdf_flutter```  隐藏目录中；\n- 将所有开发模块分支切换至与壳一致；\n- 自动分析依赖树，并**由下至上**对所有模块自动执行```flutter pub upgrade```;\n\n#### iii.开发过程中\n\n##### 1.开发模块添加\n\n- 若是有新模块需要添加入开发模块中，可直接修改initial_config.json配置文件，修改moduleNameList字段；\n- 执行tdf_tools deps更新依赖\n\n##### 2.新开发模块添加\n\n- 添加新模块后，会提示找不到模块，实际查找的是```tdf_cache```文件夹中的module_config.json文件；\n- 如果没有找到该模块，则可以执行```tdf_tools module-update```,更新远程module_config.json文件；\n- 删掉本地的module_config.json文件，重新执行命令即可，脚本会自动判断本地是否存在该配置文件，如果不存在会**下载**；\n\n<span style="color:#ff0000">请确保gitlab仓库的新开发模块master分支是一个flutter模块，如果判定不是flutter模块，则会报错（判定条件为存在pubspec.yaml文件）</span>\n\n\n\n### 3.版本控制\n\n版本控制请使用tdf_tools命令，命令详情可使用  ```tdf_tools -h```  查看，现已支持大部分命令，若有特殊命令需要执行，可以使用  ```tdf_tools <git命令>``` ，如：```tdf_tools git add .```\n\n\n\n### 4.自动打包发布\n\n暂未接入打包工具，预计下一季度进行支持；\n\n\n\n**<span style="color:#ff0000">FAQ</span>**\n\nwindows系统请使用bash命令；\n\n\n\n## 额外功能说明\n\n### 1.二维数组表达依赖树\n\n生成一个二维数组，可根据该二维数组的数据进行**并发**打tag，每一层的模块，都可以同时进行打tag发布操作，减少发布耗时；\n\n```json\n[\n\t["tdf_channel", "tdf_event", "tdf_network"], \n\t["tdf_widgets"], \n\t["tdf_smart_devices", "tdf_account_module"], \n\t["flutter_reset_module"]\n]\n```\n\n如上数据，数组中每一个节点中的模块均可同时打tag，节点之间需要由上至下的顺序进行tag操作\n\n\n\n### 2.插件更新\n\n执行 ```tdf_tools upgrade```\n\n\n\n### 3.远程模块配置文件更新\n\n执行 ```tdf_tools module module_update```\n\n\n\n## 依赖树分析原理\n\n采用有向有/无环图进行依赖树的分析\n\n数据结构采用如下：\n\n```python\nclass DependencyNode:\n    def __init__(self):\n        self.nodeName = \'\'\n        self.parent = []  # 父亲节点列表\n        self.children = []  # 子孙节点列表\n        self.delete = False\n```\n\n![dependency](./README_DIR/dependency.png)\n\n如上图1：一个正常的依赖树表示；\n\n如上图2：对图1中，依赖树所有节点去重，变换为图2有向图；\t\n\n\n\n**分析流程：**\n\n**依赖图构建**\n\n```python\n# 生成依赖图\n    def _generateDependenciesMap(self):\n        for package in self.__moduleDependenciesMap:\n            for module in moduleNameList:\n                if package == module:\n                    # 到这一步表明当前这个模块属于开发模块且在当前模块的依赖模块列表中，是当前模块的子模块\n                    self._mNodeDict[self.__moduleName].children.append(package)\n                    self._mNodeDict[package].parent.append(self.__moduleName)\n```\n\n- 共5个节点\n\n  - node构建：\n\n    - ```python\n      {\n      \t"模块1":{\n          "nodeNmae": "模块1",\n          "parent": [],\n          "children": ["模块2","模块3","模块4","模块5"],\n          "delete": False\n      \t},\n      \t"模块2":{\n          "nodeNmae": "模块2",\n          "parent": ["模块1"],\n          "children": ["模块4","模块5"],\n          "delete": False\n      \t}\n      \t"模块3":{\n          "nodeNmae": "模块3",\n          "parent": ["模块1"],\n          "children": ["模块5"],\n          "delete": False\n      \t}\n      \t"模块4":{\n          "nodeNmae": "模块4",\n          "parent": ["模块1","模块2"],\n          "children": [],\n          "delete": False\n      \t}\n      \t"模块5":{\n          "nodeNmae": "模块5",\n          "parent": ["模块1","模块2","模块3"],\n          "children": [],\n          "delete": False\n      \t}\n      }\n      ```\n\n**依赖图解析伪代码（以一维数组为例）**\n\n```python\n# 返回二维数组，用于并发打tag\n    def _generateDependenciesOrder(self):\n        resList = []\n        while 存在节点delete属性不为True:\n            \n            for：查找子节点为0的节点\n            \t设置节点delete属性为True\n              \n            for：deleteItemList = 拿到所有delete属性为true的节点\n\n            for：遍历所有节点，如果节点的子节点中包含deleteItemList的节点，则将其从子节点列表中删除\n```\n\n\n\n\n\n- **initial_config.json文件说明**\n\n  ```json\n  {\n      "featureBranch": "feature/test_dev_1", // 开发分支\n      "shellName": "flutter_reset_module",\n      // 项目需要开发的模块,可自由配置\n      "moduleNameList": [\n          "flutter_reset_module",\n          "tdf_smart_devices",\n          "tdf_widgets",\n          "tdf_channel"\n      ]\n  }\n  ```\n\n- **module_config.json文件说明**\n\n  ```json\n  {\n      "flutter_globalyun_us_module": {\n        \t"id": "11111"\n          "type": "shell",\n          "git": "git@git.2dfire.net:app/flutter/app/flutter_globalyun_us_module.git"\n      },\n      "tdf_router_anno": {\n          "type": "base",\n          "git": "git@git.2dfire.net:app/flutter/app/tdf_router_anno.git"\n      },\n  }\n  //语意\n  {\n    "模块名":{\n      "id": 项目gitlab id\n      "类型": gitlab group名\n      "git": gitlab地址\n    }\n  }\n  ```\n\n\n\n## 后续计划\n\n<span style="color:#ff0000">**问题：**</span>由于现在flutter ci 【lint】【tag】任务脚本成功率过于低，很多时候是因为项目模块的配置问题导致的，且后续会接入一键打tag工具\n\n方案：在执行统一push前，对所有模块的项目配置信息进行校验，确保数据规范；\n\n\n\n## 插件打包发布命令\n\n**插件打包命令**\n\n```\npoetry build\n```\n**插件发布命令**\n\n```\npoetry publish\n```\n\n## History\n\n### 2.0.01\n\n- Cli 框架升级；\n- 代码重构；\n\n### **1.1.00（2022-4-28）**\n\n- 国际化解决输出json中包含转义字符的问题，如\\n；\n- 四类语言输出文件自动格式化\n\n### **1.0.55（2022-4-28）**\n\n- 国际化key使用中文（依照ios项目开发形式）；\n\n### **1.0.53（2022-4-28）**\n\n- 国际化流程中，兼容解决部分json解析失败问题，譬如字符串中包含"="符号；\n\n> 错误日志如：Unterminated string starting at: line 1 column 5650 (char 5649)\n\n### **1.0.50（2022-4-28）**\n\n- 国际化增加繁体字翻译；\n\n',
    'author': 'Jian Xu',
    'author_email': '17826875951@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://git.2dfire.net/app/flutter/tools/package_tools',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9.0,<4.0.0',
}


setup(**setup_kwargs)
